package com.swmrs.controller;

import com.swmrs.enums.Tariff;
import com.swmrs.model.Bill;
import com.swmrs.model.Consumer;
import com.swmrs.model.Notification;
import com.swmrs.model.Usage;
import com.swmrs.objects.Calculator;
import com.swmrs.objects.ChargeDetail;
import com.swmrs.objects.ConsumerDashboardCounts;
import com.swmrs.objects.DashboardCounts;
import com.swmrs.objects.ReturnMessage;
import com.swmrs.objects.SmtpMailSender;
import com.swmrs.repo.BillRepo;
import com.swmrs.repo.ConsumerRepo;
import com.swmrs.repo.NotificationRepo;
import com.swmrs.repo.PaymentRepo;
import com.swmrs.repo.UsageRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.mail.MessagingException;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

@RestController
public class BillController {
    @Autowired
    private BillRepo billRepo;
    @Autowired
    private ConsumerRepo consumerRepo;
    @Autowired
    private UsageRepo usageRepo;
    @Autowired
    private PaymentRepo paymentRepo;
    @Autowired
    private NotificationRepo notificationRepo;
    @Autowired
    private SmtpMailSender smtpMailSender;

    @GetMapping(value = "/generateBill",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public ReturnMessage generateBill(){
        int counter = 0;
        try {
            Iterable<Consumer> consumers = consumerRepo.findAll();
            Timestamp today = new Timestamp(new Date().getTime());
            for (Consumer consumer : consumers) {
                String userId = consumer.getUserId();
                Timestamp lastBilled = consumer.getLastBilledDate();
                int lastReading = consumer.getLastBilledReading();
                Tariff tariff = consumer.getConnectionType();
                double pastDue = consumer.getDueAmount();
                
                    Usage lastUsage = usageRepo.findLastUsageByUserId(userId);
                    if(lastUsage != null){
                        int currentReading = lastUsage.getReading();
                        Calculator calculator = new Calculator();
                        int units = calculator.getUnits(lastReading, currentReading);
                        int days = calculator.getDays(lastBilled,today);
                        ChargeDetail paymentDetail = calculator.getPayment(units, days, tariff);
                        if(paymentDetail.getTotalBillAmount()>=0){
                            consumer.setDueAmount(pastDue+paymentDetail.getTotalBillAmount());
                            consumer.setLastBilledDate(today);
                            consumer.setLastBilledReading(currentReading);

                            Bill bill = new Bill();
                            bill.setCurrentReading(currentReading);
                            bill.setDateFrom(lastBilled);
                            bill.setDateTo(today);
                            bill.setLastReading(lastReading);
                            bill.setPreviousDue(pastDue);
                            bill.setServiceCharge(paymentDetail.getServiceCharge());
                            bill.setTotalBeforeTax(paymentDetail.getTotalBeforeTax());
                            bill.setTotalBillAmount(paymentDetail.getTotalBillAmount());
                            bill.setTotalPayable(pastDue+paymentDetail.getTotalBillAmount());
                            bill.setUsage(units);
                            bill.setUsageCharge(paymentDetail.getUsageCharge());
                            bill.setUserId(userId);
                            bill.setVat(paymentDetail.getVat());

                            consumerRepo.save(consumer);
                            billRepo.save(bill);
                            try {
                                smtpMailSender.send(consumer.getEmail(), "SWMRS Account Details",
                                        "<html>\n" +
                                                "  <head>\n" +
                                                "  </head>\n" +
                                                "  <body>\n" +
                                                "    <div id='body'>\n" +
                                                "      <p>Hi "+consumer.getFirstName()+",</p><br/>\n" +
                                                "      <p >Thank you for using SWMRS. Your bill for your usage is " +
                                                "below. If you find  any issues, don't hesitate to contact " +
                                                "administrators using this email or in any other mediums. </p>  " +
                                                "<table>\n" +
                                                "                   <tr>\n" +
                                                "                       <td>Bill Id: "+bill.getId()+" </td>|\n" +
                                                "                       <td> User Id: "+bill.getUserId()+"</td>\n" +
                                                "                   </tr>\n" +
                                                "                   <tr>\n" +
                                                "                       <td>From: "+ bill.getDateFrom()+" </td>|\n" +
                                                "                       <td>To: "+ bill.getDateTo()+"</td>\n" +
                                                "                   </tr>\n" +
                                                "                   <tr>\n" +
                                                "                       <td>Last Meter Reading: "+bill.getLastReading()+" \t&nbsp; \t&nbsp;" +
                                                "\t&nbsp;\t&nbsp; </td>|\n" +
                                                "                       <td> Current Meter Reading: "+bill.getCurrentReading()+"</td>\n" +
                                                "                   </tr>  \n" +
                                                "                   <hr/>   \n" +
                                                "                   <tr>\n" +
                                                "                       <td>Usage </td>:\n" +
                                                "                       <td>"+bill.getUsage()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td>Usage Charge </td>:\n" +
                                                "                       <td> "+bill.getUsageCharge()+"</td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td>Service Charge  </td>:\n" +
                                                "                       <td>"+bill.getServiceCharge()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td>Total Before Tax </td>:\n" +
                                                "                       <td>"+bill.getTotalBeforeTax()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td>VAT % </td>:\n" +
                                                "                       <td>"+bill.getVat()+"</td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td> Total Amount </td>:\n" +
                                                "                       <td>"+bill.getTotalBillAmount()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td> Previous Due </td>:\n" +
                                                "                       <td>"+bill.getPreviousDue()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                   <hr/> \n" +
                                                "                   <tr>\n" +
                                                "                       <td>Total Amount to pay  </td>:\n" +
                                                "                       <td> "+bill.getTotalPayable()+" </td>\n" +
                                                "                   </tr>\n" +
                                                "                </table> <br/><p>Please settle the amount within one" +
                                                " month"+
                                                "<p> Thank you </p> <br/> <p>Best Regards, </p>  " +
                                                "<p>Admin, </p><p>SWMRS</p>\n" +
                                                "    </div>\n" +
                                                "  </body>\n" +
                                                "</html>\n");
                            } catch (MessagingException e) {
                                e.printStackTrace();
                            }
                            counter++;
                        }
                    }
//                    else {
//                        System.out.println("No usage Found");
//                    }
            }
            return new ReturnMessage(counter + " Bills were generated Successfully");
        }
        catch (Exception e){
            return new ReturnMessage(e.getMessage());
        }
    }

    @GetMapping(path = "/bills",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public Iterable<Bill> bills(){
        return billRepo.findAll();
    }
    @GetMapping(path = "/getBills",produces = "application/json")
    @PreAuthorize("hasAuthority('CONSUMER')")
    public Iterable<Bill> getBills(@RequestParam String userId){
        return billRepo.findAllByUserId(userId);
    }
    @GetMapping(path = "/dashboardInfo",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public DashboardCounts dashboardInfo(){
        DecimalFormat df = new DecimalFormat("#.##");
        DashboardCounts dashboardCounts = new DashboardCounts();
        dashboardCounts.setTotalBills(billRepo.count());
        dashboardCounts.setTotalConsumers(consumerRepo.count());
        dashboardCounts.setTotalPayments(paymentRepo.count());
        dashboardCounts.setTotalUnits(billRepo.findTotalUsage()!=null?billRepo.findTotalUsage():0);
        dashboardCounts.setTotalPaymentAmount(paymentRepo.findTotalPayment()!=null?
                Double.parseDouble(df.format(paymentRepo.findTotalPayment())):0);
        dashboardCounts.setTotalBilledAmount(billRepo.findTotalBilledAmount()!=null?
                Double.parseDouble(df.format(billRepo.findTotalBilledAmount())):0);
        return dashboardCounts;
    }

    @GetMapping(path = "/getDashboardInfo",produces = "application/json")
    @PreAuthorize("hasAuthority('CONSUMER')")
    public ConsumerDashboardCounts getDashboardInfo(@RequestParam String userId){
        DecimalFormat df = new DecimalFormat("#.##");
        ConsumerDashboardCounts dashboardCounts = new ConsumerDashboardCounts();
        dashboardCounts.setTotalBills(billRepo.findCountByUserId(userId)!=null?billRepo.findCountByUserId(userId):0);
        dashboardCounts.setTotalPayments(paymentRepo.findCountByUserId(userId)!=null?
                paymentRepo.findCountByUserId(userId):0);
        dashboardCounts.setTotalUnits(billRepo.findTotalUsageByUserId(userId)!=null?
                billRepo.findTotalUsageByUserId(userId):0);
        dashboardCounts.setTotalPaymentAmount(paymentRepo.findTotalPaymentByUserId(userId)!=null?
                Double.parseDouble(df.format(paymentRepo.findTotalPaymentByUserId(userId))):0);
        dashboardCounts.setTotalBilledAmount(billRepo.findTotalBilledAmountByUserId(userId)!=null?
                Double.parseDouble(df.format(billRepo.findTotalBilledAmountByUserId(userId))):0);
        return dashboardCounts;
    }

    @GetMapping(path = "/areaConsumer",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public List<Map<String, Long>> areaConsumer(){
       return consumerRepo.countPostalCodes();
    }

    @GetMapping(path = "/areaUsage",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public List<Map<String, Long>> areaUsage(){
        return consumerRepo.totalUsagePostalCode();
    }

    @GetMapping(path = "/connectionTypeCount",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public List<Map<String, Long>> connectionType(){
        return consumerRepo.countConnectionType();
    }
    @GetMapping(path = "/notification",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public Iterable<Notification> notification(){
        return notificationRepo.latestNotification();
    }
}
