package com.swmrs.controller;

import com.swmrs.model.Consumer;
import com.swmrs.model.Payment;
import com.swmrs.objects.ReturnMessage;
import com.swmrs.repo.ConsumerRepo;
import com.swmrs.repo.PaymentRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.sql.Timestamp;
import java.util.Date;

@RestController
public class PaymentController {
    @Autowired
    private PaymentRepo paymentRepo;
    @Autowired
    private ConsumerRepo consumerRepo;

    @PostMapping(path = "/pay", produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public ReturnMessage pay(@RequestParam String userId, @RequestParam Double payment, @RequestParam Date date){
        try {
            Payment paymentInfo = new Payment();
            Consumer consumer = consumerRepo.findByUserId(userId);
            Timestamp timestamp = new Timestamp(date.getTime());
            double currentDue = consumer.getDueAmount()-payment;

            paymentInfo.setUserId(userId);
            paymentInfo.setPreviousDue(consumer.getDueAmount());
            paymentInfo.setPaymentDate(timestamp);
            paymentInfo.setPaymentAmount(payment);
            paymentInfo.setCurrentOutstanding(currentDue);

            consumer.setLastPayment(payment);
            consumer.setLastPaymentDate(timestamp);
            consumer.setDueAmount(currentDue);

            paymentRepo.save(paymentInfo);
            consumerRepo.save(consumer);

            return new ReturnMessage("Your payment recorded!");
        }
        catch (Exception e){
            return new ReturnMessage(e.getMessage());
        }
    }
    @GetMapping(path = "/getPayment",produces = "application/json")
    @PreAuthorize("hasAuthority('CONSUMER')")
    public Iterable<Payment> getPayment(@RequestParam String userId){
        return paymentRepo.findAllByUserId(userId);
    }
    @GetMapping(path = "/payments",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') ")
    public Iterable<Payment> payments(){
        return paymentRepo.findAll();
    }

    @GetMapping(path = "/hasDues",produces = "application/json")
//    @PreAuthorize("hasAuthority('ADMIN') ")
    public Iterable<Consumer> hasDues(){
        return consumerRepo.hasDues();
    }
    @GetMapping(path = "/hasExcessBalance",produces = "application/json")
//    @PreAuthorize("hasAuthority('ADMIN') ")
    public Iterable<Consumer> hasExcessBalance(){
        return consumerRepo.hasExcessBalance();
    }

}
