package com.swmrs.controller;

import com.swmrs.enums.Districts;
import com.swmrs.model.Consumer;
import com.swmrs.model.Usage;
import com.swmrs.objects.DistrictUsage;
import com.swmrs.objects.ReturnMessage;
import com.swmrs.objects.SmtpMailSender;
import com.swmrs.repo.ConsumerRepo;
import com.swmrs.repo.UsageRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.mail.MessagingException;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
public class UsageController {
    @Autowired
    private UsageRepo usageRepo;
    @Autowired
    private ConsumerRepo consumerRepo;

    @Autowired
    private SmtpMailSender smtpMailSender;

    @PostMapping(path = "/addUsage", produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public ReturnMessage addUsage(@RequestParam  String userId, @RequestParam Date timeStamp, @RequestParam int reading){
        try{
            System.out.println("time = " + timeStamp);
            Usage usage = new Usage();
            usage.setUserId(userId);
            usage.setReading(reading);
            usage.setTime(new Timestamp(timeStamp.getTime()));
            usageRepo.save(usage);
            return new ReturnMessage("Usage was successfully recorded");
        }
        catch (Exception e){
            return new ReturnMessage(e.getMessage());
        }
    }

    @GetMapping(path = "/payment-callback")
    public  void getPayent(@RequestBody RequestBody r){
        System.out.println("r = " + r);
    }

    @GetMapping(path = "/usages",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public Iterable<Usage> usages(){
        return usageRepo.findAll();
    }

    @GetMapping(path = "/getUsageByUser",produces = "application/json")
    @PreAuthorize("hasAuthority('CONSUMER')")
    public Iterable<Usage> getUsageByUser(@RequestParam String userId){
        return usageRepo.findAllByUserId(userId);
    }

    @GetMapping(path = "/usagesInInterval",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public Iterable<Usage> usagesInInterval(@RequestParam Date startDate, @RequestParam Date endDate ){
        return usageRepo.findUsageByInterval(new Timestamp(startDate.getTime()),new Timestamp(endDate.getTime()));
    }
    @GetMapping(path = "/getUsageInIntervalByUser",produces = "application/json")
    @PreAuthorize("hasAuthority('CONSUMER')")
    public Iterable<Usage> getUsageInIntervalByUser(@RequestParam  String userId, @RequestParam Date startDate,
                                       @RequestParam Date endDate ){
        return usageRepo.findUserUsageByInterval(new Timestamp(startDate.getTime()),new Timestamp(endDate.getTime()),
                userId);
    }
    @GetMapping(path = "/districtMonthlyUsage",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public List<DistrictUsage> districtMonthlyUsage(){
        List<DistrictUsage> usageDetail = new ArrayList<DistrictUsage>();
        for (Districts district: Districts.values()) {
            long sum = 0;
            Iterable<Consumer> consumers = consumerRepo.findByDistrict(district.toString());
            for (Consumer consumer : consumers) {
                Usage usage = usageRepo.findLastUsageByUserId(consumer.getUserId());
                if(usage!=null) {
                    sum = sum + usage.getReading();
                }
            }
            DistrictUsage districtUsage = new DistrictUsage();
            districtUsage.setDistrict(district.toString());
            districtUsage.setUsage(sum);
            usageDetail.add(districtUsage);
        }
        return usageDetail;
    }

    @GetMapping("/sendMail")
    @PreAuthorize("hasAuthority('ADMIN')")
    public void sendMail() {
        try {
            smtpMailSender.send("kasthuriraajan94@gmail.com", "Test mail from Spring",
                    "<html>\n" +
                            "  <head>\n" +
                            "    <style>\n" +
                            "      .colored {\n" +
                            "        color: blue;\n" +
                            "      }\n" +
                            "      #body {\n" +
                            "        font-size: 14px;\n" +
                            "      }\n" +
                            "    </style>\n" +
                            "  </head>\n" +
                            "  <body>\n" +
                            "    <div id='body'>\n" +
                            "      <p>Hi Pierce,</p>\n" +
                            "      <p class='colored'>This text is blue.</p>\n" +
                            "      <p>Jerry</p>\n" +
                            "    </div>\n" +
                            "  </body>\n" +
                            "</html>\n");
        } catch (MessagingException e) {
            e.printStackTrace();
        }
    }
}
