package com.swmrs.controller;

import com.swmrs.enums.Districts;
import com.swmrs.enums.Tariff;
import com.swmrs.enums.UserRole;
import com.swmrs.enums.UserState;
import com.swmrs.model.Consumer;
import com.swmrs.model.UserCredential;
import com.swmrs.objects.ConsumerInfo;
import com.swmrs.objects.ReturnMessage;
import com.swmrs.objects.SmtpMailSender;
import com.swmrs.repo.ConsumerRepo;
import com.swmrs.repo.UserCredentialRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.mail.MessagingException;
import java.sql.Timestamp;
import java.util.Date;

@RestController
public class UserController {

    @Autowired
    private UserCredentialRepo userCredentialRepo;
    @Autowired
    private ConsumerRepo consumerRepo;
    @Autowired
    private SmtpMailSender smtpMailSender;

    @PostMapping(path = "/addUser", produces = "application/json")
//    @PreAuthorize("hasAuthority('ADMIN')")
    public ReturnMessage addUser(@RequestParam String username){
        try {
            UserCredential userCredential = new UserCredential();
            userCredential.setUsername(username);
            userCredential.setPassword(new BCryptPasswordEncoder().encode("SamplePassword"));
            userCredential.setRole(UserRole.ADMIN);
            userCredential.setStatus(UserState.ACTIVE);
            userCredentialRepo.save(userCredential);
            return new ReturnMessage("You have added user - "+username+ "-" + "successfully");
        }
        catch (Exception e){
            return new ReturnMessage(e.getMessage());
        }
    }

    @GetMapping(path = "/showUserCredential")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public @ResponseBody
    Iterable<UserCredential> showUserCredential()
    {
        return userCredentialRepo.findAll();
    }
    @GetMapping(path = "/getUserCredential",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public UserCredential getUserCredential(@RequestParam(value = "username") String username){
        return userCredentialRepo.findByUsername(username);
    }

    @PostMapping(path = "/addConsumer", produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN')")
    public ReturnMessage addConsumer(@RequestBody ConsumerInfo consumerInfo){
        try {
            Tariff tariff = Tariff.DOMESTIC_NON_SAMURDHI;
            Consumer consumer = new Consumer(consumerInfo.getUserId(), consumerInfo.getFirstName(),
                    consumerInfo.getLastName(), consumerInfo.getEmail(), consumerInfo.getAddress(),
                    consumerInfo.getContactNo(), consumerInfo.getPostalCode(),consumerInfo.getConnectionType(),
                    consumerInfo.getDistrict());

            consumer.setLastBilledDate(new Timestamp(new Date().getTime()));

            UserCredential userCredential = new UserCredential();
            userCredential.setUsername(consumerInfo.getUserId());
            userCredential.setPassword(new BCryptPasswordEncoder().encode("SamplePassword"));
            userCredential.setRole(UserRole.CONSUMER);
            userCredential.setStatus(UserState.ACTIVE);

            consumerRepo.save(consumer);
            userCredentialRepo.save(userCredential);
            try {
                smtpMailSender.send(consumerInfo.getEmail(), "SWMRS Account Details",
                        "<html>\n" +
                                "  <head>\n" +
                                "  </head>\n" +
                                "  <body>\n" +
                                "    <div id='body'>\n" +
                                "      <p>Hi "+consumerInfo.getFirstName()+",</p><br/>\n" +
                                "      <p >Welcome to Smart Water Meter Reading System . We have created a user " +
                                "account for you. " +
                                "</p> <h3>User ID : \n" + consumerInfo.getUserId() +

                                " </h3> <h3> Password : SamplePassword </h3><br/> <p> You can use above credential to" +
                                " login to the system. You can change your password later yourself. If you find  any " +
                                "difficulties, don't hesitate to contact administrators using this email or in any " +
                                "other mediums. </p>  <p> Thank you </p> <br/> <p>Best Regards, </p>  " +
                                "<p>Admin, </p><p>SWMRS</p>\n" +
                                "    </div>\n" +
                                "  </body>\n" +
                                "</html>\n");
            } catch (MessagingException e) {
                e.printStackTrace();
            }
            return new ReturnMessage("User "+consumerInfo.getUserId()+ "-" +consumerInfo.getFirstName()+
                    " was added successfully");
        }
        catch (Exception e){
            return new ReturnMessage(e.getMessage());
        }
    }

    @GetMapping(path = "/ConsumersList")
    @PreAuthorize("hasAuthority('ADMIN')")
    public @ResponseBody
    Iterable<Consumer> consumersList()
    {
        return consumerRepo.findAll();
    }
    @GetMapping(path = "/getConsumer",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public Consumer getConsumer(@RequestParam(value = "userId") String userId){
        return consumerRepo.findByUserId(userId);
    }

    @GetMapping(path = "/consumerTypes",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public Tariff[] consumerTypes() {
        return Tariff.values();
    }

    @GetMapping(path = "/districts",produces = "application/json")
    @PreAuthorize("hasAuthority('ADMIN') or hasAuthority('CONSUMER')")
    public Districts[] districts() {
        return Districts.values();
    }

}
