package com.swmrs.objects;

import com.swmrs.enums.Tariff;

import java.sql.Timestamp;
import java.text.DecimalFormat;

public class Calculator {
    private double [] DOMESTIC_SAMURDHI_CHARGE = {5,10,15,40,58,88,105,105,120,120,130,130,130,130,130,140};
    private double [] DOMESTIC_NON_SAMURDHI_CHARGE = {8,11,20,40,58,88,105,105,120,120,130,130,130,130,130,140};
    private double [] DOMESTIC_OTHER_CHARGE = {12,16,20,40,58,88,105,105,120,120,130,130,130,130,130,140};
    private double [] SCHOOL_CHARGE = {6,6,6,6,6,6,6,6,16,16,16,16,16,16,16,16};

    private double [] DOMESTIC_SAMURDHI_SERVICE_CHARGE = {50,50,50,80,100,200,400,400,650,650,1000,1000,1000,1000,1000,1600};
    private double [] DOMESTIC_NON_SAMURDHI_SERVICE_CHARGE = {50,65,70,80,100,200,400,400,650,650,1000,1000,1000,1000,1000,1600};
    private double [] DOMESTIC_OTHER_SERVICE_CHARGE = {50,65,70,80,100,200,400,400,650,650,1000,1000,1000,1000,1000,1600};
    private double [] SCHOOL_SERVICE_CHARGE = {50,65,70,80,100,200,400,400,650,650,1000,1000,1000,1000,1000,1600};

    private double VAT = 12;


    public int getUnits(int lastReading, int currentReading){
        if(lastReading>=0 && currentReading>=lastReading){
            return (currentReading-lastReading);
        }
        else{
            return -1;
        }
    }

    public int getDays(Timestamp dateFrom, Timestamp dateTo){

        long diff = dateTo.getTime()-dateFrom.getTime();
        return (int) (diff/(1000*60*60*24));
    }

    public ChargeDetail getPayment(int units, int days, Tariff tariff){
        ChargeDetail chargeDetail = new ChargeDetail();
        DecimalFormat df = new DecimalFormat("#.##");
        if((units>=0)&&(days>0)&&(tariff!=null)){
            double [] unitCharge;
            double [] serviceCharge;
            double payment= 0;
            switch (tariff){
                case DOMESTIC_SAMURDHI:
                    unitCharge = DOMESTIC_SAMURDHI_CHARGE;
                    serviceCharge = DOMESTIC_SAMURDHI_SERVICE_CHARGE;
                    break;
                case DOMESTIC_NON_SAMURDHI:
                    unitCharge = DOMESTIC_NON_SAMURDHI_CHARGE;
                    serviceCharge = DOMESTIC_NON_SAMURDHI_SERVICE_CHARGE;
                    break;
                case DOMESTIC_OTHER:
                    unitCharge = DOMESTIC_OTHER_CHARGE;
                    serviceCharge = DOMESTIC_OTHER_SERVICE_CHARGE;
                    break;
                default:
                    unitCharge = SCHOOL_CHARGE;
                    serviceCharge = SCHOOL_SERVICE_CHARGE;
                    break;
            }

            int reminder = units/5;
            if(reminder>15){
                reminder =15;
            }
            double calcUnit = (double)days/30;
            for (int i=0;i<reminder;i++){
                payment = payment + (unitCharge[i]* (calcUnit*5));
            }
            double remain = units-(calcUnit*reminder);
            payment = payment + remain*unitCharge[reminder];
            chargeDetail.setUsageCharge(Double.parseDouble(df.format(payment)));
            chargeDetail.setServiceCharge(Double.parseDouble(df.format(serviceCharge[reminder])));

            payment = payment +serviceCharge[reminder];
            chargeDetail.setTotalBeforeTax(Double.parseDouble(df.format(payment)));

            chargeDetail.setVat(VAT);
            payment = payment+ ((payment*VAT)/100);
            chargeDetail.setTotalBillAmount(Double.parseDouble(df.format(payment)));
            return chargeDetail;
        }
        else {
            chargeDetail.setTotalBillAmount(-1);
            return chargeDetail;
        }
    }

}
